#include "drawable.h"

#include <uta/painter.h>
#include <uta/resources.h>

using namespace Janus;
using namespace uta;

class DrawingWidget : public Widget
{
public:
  DrawingWidget(Widget* parent, int x, int y, int w, int h):
    Widget(parent,x,y,w,h) 
 {}
  virtual ~DrawingWidget() {};
  SigC::Signal1<bool, const uta::Event*> eventSignal;
  Point getOffset() {  return globalCoord(window_).upperLeft(); };
protected:
  virtual bool processEvent(const uta::Event* event) 
  { if (mouseFocus_) return eventSignal.emit(event); else return false; }; 
};


Drawable::Drawable(const Parameters& param) :
  WidgetBase(param),
  drawingFont_(NULL)
{
  widget_ = new DrawingWidget(parent_, 
			      rect_.upperLeft().x, rect_.upperLeft().y,
			      rect_.width(), rect_.height());
  
  assert(widget_); 
  widget_->setAutoDelete(false);
  
  surface_ = new uta::Surface(rect_.width(), rect_.height());
  surface_->fill(uta::white);

  widget_->setBackground(surface_);  

  std::string filter = Element::getStringAttr("filter","");
  if(!filter.empty())
    {
      ((DrawingWidget*)widget_)
	->eventSignal.connect(SigC::slot(this, &Drawable::eventHandler));

      filterMove_ = (filter.find("move") != std::string::npos);
      filterClick_ = (filter.find("click") != std::string::npos);
      filterKeys_ = (filter.find("key") != std::string::npos);
      
    }
}

Drawable::~Drawable()
{
  delete surface_;
}

bool
Drawable::eventHandler(const uta::Event* event)
{
  uta::Point offset = ((DrawingWidget*)widget_)->getOffset();

  if(filterClick_ && event->type() == uta::Event::BUTTON)
    {        
      
      uta::Point pos = ((uta::ButtonEvent *)event)->position()-offset;
      char buffer[128];
      sprintf(buffer,"%d %d",pos.x, pos.y);
      switch(((uta::ButtonEvent*)event)->buttons())
	{
	case uta::ButtonEvent::BUTTON_LEFT:
	  emit("button1", buffer);
	  break;
	case uta::ButtonEvent::BUTTON_MIDDLE:
	  emit("button2", buffer);
	  break;
	case uta::ButtonEvent::BUTTON_RIGHT:
	  emit("button3", buffer);
	  break;
	}	  
      
    }
  if(filterMove_ && event->type() == uta::Event::MOUSE)
    {
      uta::Point pos = ((uta::MouseEvent *)event)->position()-offset;
      char buffer[128];
      sprintf(buffer,"%d %d",pos.x, pos.y);
      emit("move", buffer);
    }
  if(filterKeys_ &&  event->type() == uta::Event::KEY)
    {
      std::string action;
      
//       if(((KeyEvent *)event)->state() == KeyEvent::PRESSED)
// 	action = "pressed";
//       else
// 	action = "released";

      char input = ((uta::KeyEvent *)event)->ascii();
      
      emit("key", std::string(1, input));
    }
  return false;
}

void 
Drawable::call(const std::string& name, const Value& val)
{ 
  if(name == "clear")
    {
      surface_->clear();
      widget_->setBackground(surface_);
    }
  else if(name == "setColor")
    {
      int r,g,b;
      sscanf(val.c_str(),"%d %d %d",&r,&g,&b);
      drawingColor_ = uta::Color(r,g,b);
    }
  else if(name == "setColor2")
    {
      int r,g,b;
      sscanf(val.c_str(),"%d %d %d",&r,&g,&b);
      drawingColor2_ = uta::Color(r,g,b);
    }  
  else if(name == "setColor3")
    {
      int r,g,b;
      sscanf(val.c_str(),"%d %d %d",&r,&g,&b);
      drawingColor3_ = uta::Color(r,g,b);
    }
  else if(name == "line")
    {
      int x1,y1,x2,y2;
      sscanf(val.c_str(),"%d %d %d %d",&x1,&y1,&x2,&y2);
      uta::Painter* painter = new uta::Painter(surface_);
      painter->setColor(drawingColor_);
      painter->line(uta::Point(x1,y1),uta::Point(x2,y2));
      delete painter;
      
      widget_->setBackground(surface_);
    }
  else if(name == "point")
    {
      int x1,y1;
      sscanf(val.c_str(),"%d %d",&x1,&y1);
      uta::Painter* painter = new uta::Painter(surface_);
      painter->setPixel(uta::Point(x1,y1), drawingColor_);
      delete painter;
      
      widget_->setBackground(surface_);
    }
  else if(name == "ellipse")
    {
      int x,y,rx,ry;
      sscanf(val.c_str(),"%d %d %d %d",&x,&y,&rx,&ry);
      uta::Painter* painter = new uta::Painter(surface_);
      painter->setColor(drawingColor_);
      painter->ellipse(uta::Point(x,y),rx,ry);
      delete painter;
      
      widget_->setBackground(surface_);
    }
  else if(name == "box")
    {
      int x1,y1,x2,y2;
      sscanf(val.c_str(),"%d %d %d %d",&x1,&y1,&x2,&y2);
      uta::Painter* painter = new uta::Painter(surface_);
      painter->setColor(drawingColor_);
      painter->box(uta::Point(x1,y1),uta::Point(x2,y2));
      delete painter;
      
      widget_->setBackground(surface_);
    }
  else if(name == "shaded_trigon")
    {
      int x1,y1,x2,y2,x3,y3;
      sscanf(val.c_str(),"%d %d %d %d %d %d",&x1,&y1,&x2,&y2,&x3,&y3);
      uta::Painter* painter = new uta::Painter(surface_);
      painter->shadedTrigon(uta::Point(x1,y1),uta::Point(x2,y2), 
			    uta::Point(x3,y3),
			    drawingColor_, drawingColor2_, drawingColor3_);
      delete painter;
      
      widget_->setBackground(surface_);
    }
  else if(name == "setFont")
    {
      if(RES_AVAILABLE(val))
	drawingFont_ = FONT_RES(val);
      else
	drawingFont_ = NULL;
    }
  else if(name == "text")
    {
      if(drawingFont_)
	{
	  int x1, y1;
	  char text[1024];
	  sscanf(val.c_str(),"%d %d %s",&x1,&y1,text);
	  uta::Surface* tmp = drawingFont_->getString(text);
	  std::vector<Color> palette;
	  palette.resize(5);

	  int rdiff = drawingColor_.r - drawingColor2_.r;
	  int gdiff = drawingColor_.g - drawingColor2_.g;
	  int bdiff = drawingColor_.b - drawingColor2_.b;
	  int adiff = drawingColor_.a - drawingColor2_.a;
	  
	  int r = drawingColor2_.r;
	  int g = drawingColor2_.g;
	  int b = drawingColor2_.b;
	  int a = drawingColor2_.a;
	  
	  palette[0] = transparent;
	  
	  for(unsigned char num=1; num < 5; num++)
	    palette[num]= Color(r + num * rdiff/4,
				 g + num * gdiff/4,
				 b + num * bdiff/4,
				 a + num * adiff/4);
	  
	  tmp->setPalette(palette);
	  
	  tmp->blit(surface_,Rect(x1,y1,tmp->width(),tmp->height()));
	  
	  delete tmp;
	  widget_->setBackground(surface_);
	}
    }	  
  else
    {
      WidgetBase::call(name, val);
    }
}
  

