//  The Worldforge Project
//  Copyright (C) 1998, 1999 The Worldforge Project
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//  For information about Worldforge and its authors, please contact
//  the Worldforge Web Site at http://www.worldforge.org.
//
// created by Karsten Laux 11/2000

#include <string>
#include <map>
#if __GNUC__ < 3
#include <hash_map>
#else
#include <ext/hash_map>
#endif

#include "values.h"
#include "callback.h"
#include "factory.h"

#ifndef _JANUS_ELEMENT_H_
#define _JANUS_ELEMENT_H_

namespace Janus {


/** Dynamic Janus Base Element.
 *   Element itself carries just the ID. It is up to derived classes to 
 *  make use of the parent and attributes member.
 *  It may emit named events and in return from outside named methods of Element
 *  can be called.
 *  The Factory concepts makes the including of all definitions superflucious.
 *  Thus any implementations of Elements may be very GUI specific, but the Janus engine
 *  does not care, as it includes only this base definition.
 *  
 *  Element::Directory is a map of all elements of this dialog 
 *
 */
class Element
{ 
 public: 

  ///
  struct eqstr
  {
    bool operator()(const std::string& s1, const std::string& s2) const
    {
      return strcmp(s1.c_str(), s2.c_str()) == 0;
    }
  };

  ///
  struct hasher
  {
    size_t operator () (const std::string& s) const
    {
      return __gnu_cxx::hash<const char*>()(s.c_str());
    }
  };

  /** Element Directory.
  */
  typedef __gnu_cxx::hash_map<std::string, Element*, hasher, eqstr> Directory;

  ///
  typedef struct 
  {
    std::string id;
    Element* parent;
    Values values;
    Directory* directory;
  } Parameters;

  /** register new factory.
      Element takes over ownership for the factory !
  */
  static void registerFactory(const std::string&, Janus::Factory<Element>*);
  /** Create element of given type.
   *  \return pointer to new object or NULL if no factory for requested type
   *  is available.
   */
  static Element* createElement(const std::string& type, const Element::Parameters&);
  ///
  static void cleanup();

  ///
  template <typename T> class Factory : public Janus::Factory<Element>
    {
    public:
      Factory(const std::string& type): Janus::Factory<Element>() 
	{ 
	  Element::registerFactory(type, this); 
	};

      virtual ~Factory() {}
      
      virtual Element* New(const Parameters& p)
        {
	  //create new Element and add it to the Directory
	  T* element =  new T(p);
	  (*(p.directory))[p.id] = element;
	  return element;
        }
 
        virtual void Delete(Element* element)
        {
	  delete element;
        }
    };    
 
  /** Destructor.
      remove this Element from directory.
  */
  virtual ~Element();

  ///
  std::string id() const { return id_; };
  
  ///
  virtual Value getValue(void) const { return VALUE_NONE;};
  ///
  virtual void setValue(const Value&) {};

  /// set attribute
  virtual void setAttr(const std::string& id, const Value& attr) 
    { 
      attributes_[id] = attr; 
    };
  
  ///
  virtual const Value& getAttr(const std::string& id, const std::string& fallback="") const 
    { 
      Values::const_iterator itr = attributes_.find(id);
      if(itr == attributes_.end())
	return fallback;
      return itr->second;
    }
  
  ///
  virtual std::string getStringAttr(const std::string& id, const std::string& fallback="") const
    {
      return getAttr(id, fallback);
    }

  ///
  virtual int getIntAttr(const std::string& id, int fallback=0) const
    {
      Values::const_iterator itr = attributes_.find(id);
      if(itr == attributes_.end())
	return fallback;
      return atoi(itr->second.c_str());     
    }
  ///
  virtual float getFloatAttr(const std::string& id, float fallback=0.0) const
    {
     Values::const_iterator itr = attributes_.find(id);
      if(itr == attributes_.end())
	return fallback;
      return atof(itr->second.c_str());
    }
  ///
  bool hasAttr(const std::string& id) const 
    {
      return attributes_.find(id) != attributes_.end();
    };

  ///
  const Values& attributes() const { return attributes_; };

  ///
  Element* parent() const { return parent_; };

  /** call named method.
   */
  virtual void call(const std::string& name, const Value& val = VALUE_NONE)
    { if(cb_) cb_->call(id_, name, val); } ;
  
  ///
  void setCallback(Callback* cb) { cb_ = cb; };

 protected:
  /** Constructor.
      add this Element to the global element directory.
  */
  Element(const Parameters& param);
  
  /** emit named event.
   */
  virtual void emit(const std::string&, const Value&) const {};

  ///
  std::string id_;
  ///
  Element* parent_;
  ///
  Values attributes_;
  ///where to look for my friends and where friends look for me
  Directory* directory_;
  ///
  Callback* cb_;

  ///
  static std::map<std::string, Janus::Factory<Element> * > *factories_;
};


} // namespace Janus

#endif
  
